@@ -34,75 +34,110 @@ class MaintenanceManager(Manager):
         """
             Create new Personalize solution versions and update Personalize campaigns
             when required
+
+            Return:
+                (bool) : the outcome of maintenance "updated", "not_updated", "blocked"
         """
-        # Get all active client subscriptions for the supplied service
-        for client in self.query_helper.get_clients():
-            # Set up a flag to decide whether its necessary to update this subscription
-            # after maintenance of this subscription
-            update = False
+        maintenance_outcome = "not_updated"
 
-            subscription = self.query_helper.get_subscription(
-                client_id=client.id,
-                service_id=self.service.id,
-                from_cache=False,
-                for_update=True
-            )
+        # check if maintenance can be performed
+        self.can_perform_maintenance = self.check_maintenance_is_allowed()
 
-            solutions, engine_index = self.query_helper.get_engine_entity(
-                client_id=client.id,
-                service_id=self.service.id,
-                engine_slug=self.engine_slug,
-                entity_key='solutions',
-                from_cache=False
-            )
+        if self.can_perform_maintenance:
+            # Get all active client subscriptions for the supplied service
+            for client in self.query_helper.get_clients():
+                # Set up a flag to decide whether its necessary to update this subscription
+                # after maintenance of this subscription
+                update = False
 
-            if solutions is not None:
-                response = self.perform_solution_maintenance(
-                    client=client,
-                    solutions=solutions
+                maintenance_mode, _ = self.query_helper.get_engine_entity(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=self.engine_slug,
+                    entity_key='maintenance_mode',
+                    from_cache=False
                 )
 
-                if response.get('solutions_updated') is True:
-                    self.query_helper.update_solutions(
-                        subscription=subscription,
-                        engine_index=engine_index,
-                        value=response.get('solutions')
-                    )
-                    update = True
+                if maintenance_mode is None or maintenance_mode == 'off':
+                    continue
 
-            campaigns, engine_index = self.query_helper.get_engine_entity(
-                client_id=client.id,
-                service_id=self.service.id,
-                engine_slug=self.engine_slug,
-                entity_key='campaigns',
-                from_cache=False
-            )
-
-            if campaigns is not None:
-                response = self.perform_campaign_maintenance(
-                    client=client,
-                    campaigns=campaigns
+                subscription = self.query_helper.get_subscription(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    from_cache=False,
+                    for_update=True
                 )
 
-                if response.get('campaigns_updated') is True:
-                    self.query_helper.update_campaigns(
-                        subscription=subscription,
-                        engine_index=engine_index,
-                        value=response.get('campaigns')
+                solutions, engine_index = self.query_helper.get_engine_entity(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=self.engine_slug,
+                    entity_key='solutions',
+                    from_cache=False
+                )
+
+                if solutions is not None:
+                    response = self.perform_solution_maintenance(
+                        client=client,
+                        solutions=solutions,
+                        maintenance_mode=maintenance_mode
                     )
-                    update = True
 
-            if update:
-                self.query_helper.update(model=subscription)
-                self.query_helper.flush_subscription_properties_cache(subscription=subscription)
+                    if response.get('solutions_updated') is True:
+                        self.query_helper.update_solutions(
+                            subscription=subscription,
+                            engine_index=engine_index,
+                            value=response.get('solutions')
+                        )
+                        update = True
 
-    def perform_solution_maintenance(self, *, client: Client, solutions: list) -> dict:
+                campaigns, engine_index = self.query_helper.get_engine_entity(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=self.engine_slug,
+                    entity_key='campaigns',
+                    from_cache=False
+                )
+
+                if campaigns is not None:
+                    response = self.perform_campaign_maintenance(
+                        client=client,
+                        campaigns=campaigns,
+                        maintenance_mode=maintenance_mode
+                    )
+
+                    if response.get('campaigns_updated') is True:
+                        self.query_helper.update_campaigns(
+                            subscription=subscription,
+                            engine_index=engine_index,
+                            value=response.get('campaigns')
+                        )
+                        update = True
+
+                if update:
+                    self.query_helper.update(model=subscription)
+                    self.query_helper.flush_subscription_properties_cache(subscription=subscription)
+                    maintenance_outcome = "updated"
+        else:
+            maintenance_outcome = "blocked"
+
+        return maintenance_outcome
+
+    def perform_solution_maintenance(
+            self,
+            *,
+            client: Client,
+            solutions: list,
+            maintenance_mode: str
+    ) -> dict:
         """
             Perform the required maintenance on a list of solutions
 
             args:
                 client (Client): The client this maintenance is being performed for
                 solutions (list): A list of solutions from the database to perform maintenance on
+                maintenance_mode (str): The maintenance mode of the subscription
+                                        (full/database_only)
 
             returns:
                 result (dict): A dictionary with an updated flag and resultant solutions
@@ -122,7 +157,7 @@ class MaintenanceManager(Manager):
                 solution=solution
             )
 
-            if state.get('should_create_version') is True:
+            if maintenance_mode == 'full' and state.get('should_create_version') is True:
                 print(
                     'Building solution version for client: {client_name}'
                     .format(client_name=client.name)
@@ -169,13 +204,21 @@ class MaintenanceManager(Manager):
             'solutions': solutions
         }
 
-    def perform_campaign_maintenance(self, *, client: Client, campaigns: list) -> dict:
+    def perform_campaign_maintenance(
+            self,
+            *,
+            client: Client,
+            campaigns: list,
+            maintenance_mode: str
+    ) -> dict:
         """
             Perform the required maintenance on a list of campaigns
 
             args:
                 client (Client): The client this maintenance is being performed for
                 campaigns (list): A list of campaigns from the database to perform maintenance on
+                maintenance_mode (str): The maintenance mode of the subscription
+                                        (full/database_only)
 
             returns:
                 result (dict): Dictionary containing:
@@ -201,7 +244,7 @@ class MaintenanceManager(Manager):
                 campaign=campaign
             )
 
-            if state.get('should_update') is True:
+            if maintenance_mode == 'full' and state.get('should_update') is True:
                 latest_solution_version_arn = state.get(
                     'latest_solution_version_arn'
                 )
