@@ -52,16 +52,16 @@ class DatasetStateManager:
             'Failed',
             'RollingBack'],
         'stopped':['OutOfService'],
-        'active':['InService']
+        'active':['InService', None, 'None']
     }
 
     solution_statuses = {
         'blocking':['InProgress', 'Stopping', 'Failed', 'Stopped'],
-        'success':['Completed']
+        'success':['Completed', None, 'None']
     }
 
     dataset_statuses = {
-        'active': ['Active'],
+        'active': ['Active', None, 'None'],
         'blocking': ['Updating']
     }
 
@@ -116,17 +116,18 @@ class DatasetStateManager:
 
     @staticmethod
     def parse_s3_url(url: str):
+        """Returns an (s3 bucket, key name/prefix) tuple from a url with an s3
+        scheme.
+        Args:
+            url (str):
+        Returns:
+            tuple: A tuple containing:
+                str: S3 bucket name str: S3 key
         """
-            Parse S3URL to find bucket name and filepath
-
-            args:
-                url (str): string s3:// url
-            return:
-                (bucket-name (str), s3 filepath (str))
-        """
-        url_obj = urlparse(url, allow_fragments=False)
-
-        return url_obj.netloc, url_obj.path
+        parsed_url = urlparse(url)
+        if parsed_url.scheme != "s3":
+            raise ValueError("Expecting 's3' scheme, got: {} in {}.".format(parsed_url.scheme, url))
+        return parsed_url.netloc, parsed_url.path.lstrip("/")
 
     def upload_lookups(self, s3_url: str):
         """Upload dataset from s3"""
