@@ -30,10 +30,13 @@ class MaintenanceManager:
     """
         Manage AWS SDC Engine solutions and campaigns
     """
+    # pylint: disable=too-many-instance-attributes
+    # flag to determine if maintenance was performed
+    current_client = None
+    can_perform_maintenance = False
     # get helpers
     date_utils = DateUtils()
     slack_helper = SlackHelper()
-    solution_base_name = None
 
     # set up states
     solution_job_status = {
@@ -42,7 +45,7 @@ class MaintenanceManager:
 
     def __init__(self, **kwargs):
         # get service slug
-        service_slug = kwargs.get('service_slug', 'recommend')
+        self.service_slug = kwargs.get('service_slug', 'recommend')
         # get rds config
         rds_config = kwargs.get('rds_config', {})
         if not isinstance(rds_config, dict):
@@ -57,9 +60,13 @@ class MaintenanceManager:
         # instantiate query helpers
         self.query_helper = SagemakerEngineHelpers(
             rds_config=rds_config,
-            redis_config=redis_config)
+            redis_config=redis_config
+        )
         # get service config for service slug
-        self.service = self.query_helper.get_service(slug=service_slug)
+        self.service = self.query_helper.get_service(
+            slug=self.service_slug,
+            for_update=True
+        )
         # create instance of sagemaker
         self.sagemaker = boto3.client('sagemaker')
         # init state managers
@@ -68,50 +75,118 @@ class MaintenanceManager:
         self.dataset_state_manager = DatasetStateManager()
 
     def __del__(self):
+        # finally close query helper
+        if self.can_perform_maintenance:
+            # reset maintenance_in_progress
+            self.service = self.query_helper.get_service(
+                slug=self.service_slug,
+                for_update=True
+            )
+
+            self.service.properties['maintenance_in_progress'] = None
+            # update service
+            self.query_helper.update(
+                model=self.service
+            )
         del self.query_helper
 
+    def check_maintenance_is_allowed(self) -> bool:
+        """
+            checks if maintenance is currently in progress for a given service.
+            if not, returns True allowing the current manager to take ownership and run
+            maintenance else returns False to exit gracefully.
+
+            Returns:
+                bool: Whether current manager can perform maintenance.
+        """
+        allow_maintenance = False
+
+        if (
+                isinstance(self.service.properties, dict) and
+                self.service.properties.get('maintenance_in_progress', None) is None
+        ):
+            allow_maintenance = True
+        elif (
+                self.service.properties is None
+        ):
+            # setting empty properties field as a dict in python model
+            self.service.properties = {}
+            allow_maintenance = True
+
+        if allow_maintenance:
+            # no maintenance in progress
+            # run maintenance
+            self.service.properties['maintenance_in_progress'] = self.engine_slug
+
+            self.query_helper.update(
+                model=self.service
+            )
+
+        return allow_maintenance
+
     def perform_maintenance(self):
         """
             Execute maintenance that performs a set of tasks for engines
             These are specifically Update, refresh or do nothing.
 
             Return:
-                (bool) : whether any updates where performed
+                (bool) : the outcome of maintenance "updated", "not_updated", "blocked"
         """
-        database_session_updated = False
+        maintenance_outcome = "not_updated"
 
-        for client in self.query_helper.get_clients():
+        # check if maintenance can be performed
+        self.can_perform_maintenance = self.check_maintenance_is_allowed()
 
-            # get subscription for engine
-            subscription = self.query_helper.get_subscription(
-                client_id=client.id,
-                service_id=self.service.id,
-                from_cache=False,
-                for_update=True
-            )
-
-            solution_updated = self.perform_solution_maintenance(
-                client=client,
-                engine_slug=self.engine_slug
-            )
-
-            campaign_updated = self.perform_campaign_maintenance(
-                client=client,
-                engine_slug=self.engine_slug
-            )
-
-            datasets_updated = self.perform_dataset_maintenance(
-                client=client,
-                engine_slug=self.engine_slug
-            )
-
-            if any([solution_updated, campaign_updated, datasets_updated]):
-                # when any of the above have been updated
-                database_session_updated = self.perform_session_update(
-                    subscription=subscription
+        if self.can_perform_maintenance:
+            # start maintenance
+            for client in self.query_helper.get_clients():
+                maintenance_mode, _ = self.query_helper.get_engine_entity(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=self.engine_slug,
+                    entity_key='maintenance_mode',
+                    from_cache=False
                 )
 
-        return database_session_updated
+                if maintenance_mode is None or maintenance_mode == 'off':
+                    continue
+
+                # get subscription for engine
+                subscription = self.query_helper.get_subscription(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    from_cache=False,
+                    for_update=True
+                )
+
+                self.current_client = client
+                solution_updated = self.perform_solution_maintenance(
+                    client=client,
+                    engine_slug=self.engine_slug,
+                    maintenance_mode=maintenance_mode
+                )
+
+                campaign_updated = self.perform_campaign_maintenance(
+                    client=client,
+                    engine_slug=self.engine_slug,
+                    maintenance_mode=maintenance_mode
+                )
+
+                datasets_updated = self.perform_dataset_maintenance(
+                    client=client,
+                    engine_slug=self.engine_slug,
+                    maintenance_mode=maintenance_mode
+                )
+
+                if any([solution_updated, campaign_updated, datasets_updated]):
+                    # when any of the above have been updated
+                    maintenance_outcome = self.perform_session_update(
+                        subscription=subscription
+                    )
+        else:
+            maintenance_outcome = "blocked"
+
+        return maintenance_outcome
 
     @engine_decorators.retry_handler(
         exceptions=(Exception, InternalError),
@@ -126,15 +201,16 @@ class MaintenanceManager:
             Returns:
                 updated (bool): whether the database session was updated
         """
+        print("Updating subscription")
         self.query_helper.update(
             model=subscription
         )
-
+        print("clearing cache")
         self.query_helper.flush_subscription_properties_cache(
             subscription=subscription
         )
 
-        return True
+        return "updated"
 
 
     @global_decorators.general_exception_handler(
@@ -145,7 +221,9 @@ class MaintenanceManager:
     def perform_dataset_maintenance(
             self, *,
             client: dict,
-            engine_slug: str) -> dict:
+            engine_slug: str,
+            maintenance_mode: str
+    ) -> dict:
         """
             Performs dataset update maintainence
 
@@ -154,71 +232,72 @@ class MaintenanceManager:
         """
         datasets_updated = False
         curr_state = {}
-        # get datasets
-        datasets, _ = self.query_helper.get_engine_entity(
-            client_id=client.id,
-            service_id=self.service.id,
-            engine_slug=engine_slug,
-            entity_key='datasets',
-            from_cache=False,
-            for_update=True
-        )
-
-        if datasets is None:
-            return datasets_updated
-
-        for _, dataset in enumerate(datasets):
-
-            label = dataset.get('label', None)
-            if label is None:
-                # skip this dataset
-                continue
-
-            new_dataset = {}
-
-            # get solution from db
-            campaign, _ = self.query_helper.get_campaign(
+        if maintenance_mode != 'off':
+            # get datasets
+            datasets, _ = self.query_helper.get_engine_entity(
                 client_id=client.id,
                 service_id=self.service.id,
                 engine_slug=engine_slug,
+                entity_key='datasets',
                 from_cache=False,
-                for_update=True,
-                recipe=dataset['recipe']
-            )
-            # get dataset
-            curr_state = self.dataset_state_manager.get_state(
-                dataset=dataset,
-                campaign=campaign
+                for_update=True
             )
 
-            if curr_state.get('should_update') is True:
+            if datasets is None:
+                return datasets_updated
 
-                print(
-                    'Updating dataset version for client: {client_name}'
-                    .format(client_name=client.name)
+            for _, dataset in enumerate(datasets):
+
+                label = dataset.get('label', None)
+                if label is None:
+                    # skip this dataset
+                    continue
+
+                new_dataset = {}
+
+                # get solution from db
+                campaign, _ = self.query_helper.get_campaign(
+                    client_id=client.id,
+                    service_id=self.service.id,
+                    engine_slug=engine_slug,
+                    from_cache=False,
+                    for_update=True,
+                    recipe=dataset['recipe']
                 )
-
-                new_dataset = self.dataset_state_manager.update_state(
+                # get dataset
+                curr_state = self.dataset_state_manager.get_state(
                     dataset=dataset,
                     campaign=campaign
                 )
 
-                # update dataset
-                self.query_helper.update_dataset(
-                    client_id=client.id,
-                    service_id=self.service.id,
-                    engine_slug=engine_slug,
-                    label=dataset['label'],
-                    config=new_dataset
-                )
+                if curr_state.get('should_update') is True:
 
-                dataset_label = dataset['label']
-                print(
-                    'Successfully updated dataset version: {dataset_label}'
-                    .format(dataset_label=dataset_label)
-                )
+                    print(
+                        'Updating dataset version for client: {client_name}'
+                        .format(client_name=client.name)
+                    )
 
-                datasets_updated = True
+                    new_dataset = self.dataset_state_manager.update_state(
+                        dataset=dataset,
+                        campaign=campaign
+                    )
+
+                    # update dataset
+                    self.query_helper.update_dataset(
+                        client_id=client.id,
+                        service_id=self.service.id,
+                        engine_slug=engine_slug,
+                        label=dataset['label'],
+                        config=new_dataset
+                    )
+
+                    dataset_label = dataset['label']
+                    print(
+                        'Successfully updated dataset version: {dataset_label}'
+                        .format(dataset_label=dataset_label)
+                    )
+
+                    datasets_updated = True
 
         return datasets_updated
 
@@ -228,13 +307,21 @@ class MaintenanceManager:
         exceptions=(botocore_clienterror, Exception),
         should_raise=False
     )
-    def perform_solution_maintenance(self, *, client: dict, engine_slug: str) -> dict:
+    def perform_solution_maintenance(
+            self,
+            *,
+            client: dict,
+            engine_slug: str,
+            maintenance_mode: str
+    ) -> dict:
         """
             Perform the required maintenance on a list of solutions
 
             args:
                 client (dict): The client this maintenance is being performed for
-                solutions (list): A list of solutions from the database to perform maintenance on
+                engine_slug (str): Slug of the engine involved
+                maintenance_mode (str): The maintenance mode of the subscription
+                                        (full/database_only)
 
             returns:
                 result (dict): A dictionary with an updated flag and resultant solutions
@@ -268,7 +355,7 @@ class MaintenanceManager:
                 solution=solution
             )
 
-            if curr_state.get('should_create_version') is True:
+            if maintenance_mode == 'full' and curr_state.get('should_create_version') is True:
                 print(
                     'Building solution version for client: {client_name}'
                     .format(client_name=client.name)
@@ -342,13 +429,17 @@ class MaintenanceManager:
     def perform_campaign_maintenance(
             self, *,
             client: dict,
-            engine_slug: str) -> dict:
+            engine_slug: str,
+            maintenance_mode: str
+    ) -> dict:
         """
             Perform the required maintenance on a list of campaigns
 
             args:
                 client (dict): The client this maintenance is being performed for
-                campaigns (list): A list of campaigns from the database to perform maintenance on
+                engine_slug (str): Slug of the engine involved
+                maintenance_mode (str): The maintenance mode of the subscription
+                                        (full/database_only)
 
             returns:
                 result (dict): Dictionary containing:
@@ -393,7 +484,7 @@ class MaintenanceManager:
                 solution=solution
             )
 
-            if curr_state.get('should_update') is True:
+            if maintenance_mode == 'full' and curr_state.get('should_update') is True:
 
                 # update campaign state >> campaign_state_manager
                 latest_solution_version_arn = curr_state.get(
