@@ -46,8 +46,8 @@ class RecommendationsManager:
         service_slug = kwargs.pop('service_slug', 'recommend')
 
         # get name of lookups
-        self.item_id_identifier = kwargs.pop('item_id_identifier', 'item_ids')
-        self.item_code_identifier = kwargs.pop('item_code_identifier', 'item_codes')
+        self.item_id_identifier = kwargs.pop('item_id_identifier', 'item_id_lookup')
+        self.item_code_identifier = kwargs.pop('item_code_identifier', 'item_code_lookup')
 
         # load service details from service table
         self.service = self.query_helper.get_service(slug=service_slug)
